@@ -1,8 +1,7 @@
 import numpy as np
 import pandas as pd
-import matplotlib.pyplot as plt
-import matplotlib as mpl
 import seaborn as sns
+
 from sklearn.metrics import (
     auc,
     precision_recall_curve,
@@ -16,50 +15,114 @@ from sklearn.metrics import (
     roc_curve,
 )
 
-from IPython.core.display import display, HTML
-import warnings
-
-sns.set_style("ticks")
-mpl.rcParams["axes.linewidth"] = 2
-mpl.rcParams["lines.linewidth"] = 2
-warnings.filterwarnings("ignore")
-display(HTML("<style>.container { width:95% !important; }</style>"))
+from IPython.core.display import display
+from slickml.plotting import plot_binary_classification_metrics
 
 
 class BinaryClassificationMetrics:
-    """Binary Classification Metrics --------------------------
-    Parameters:
-               - y_true: list[int] List of ground truth binary values [0, 1]
-               - y_pred_proba: list[float] List of predicted probability for
-               the positive class (class=1 or y_pred_proba[:, 1] in
-               scikit-learn)
-               - threshold: float, optional (default=0.5) Threshold value for
-               mapping y_pred_prob to y_pred (> is used not >=)
-               - average_method: string, optional (default="binary") Method to
-               calculate the average of the metric. Possible values are
-               "micro", "macro", "weighted", "binary"
-               - precision_digits: int, optional (default=3) The number of
-               precision digits to display scores in final dataframe
-               - figsize: tuple, optional (default=(12,12)) Figure size for
-               scores vs thresholds plots
-               - display_df: boolean, optional (default=True) Flag to display
-               metrics DataFrame with CSS formatting
-
+    """Binary Classification Metrics.
+    This is wrapper to calculate all the binary classification
+    metrics with both arbitrary and three computed methods for
+    calculating the thresholds. Threshold computations including:
+    1) Youden Index: (https://en.wikipedia.org/wiki/Youden%27s_J_statistic).
+    2) Maximizing Precision-Recall.
+    3) Maximizing Sensitivity-Specificity.
+    Parameters
+    ----------
+    y_true: numpy.array[int] or list[int]
+        List of ground truth binary values [0, 1]
+    y_pred_proba: numpy.array[float] or list[float]
+        List of predicted probability for the positive class
+        (class=1 or y_pred_proba[:, 1] in scikit-learn)
+    threshold: float, optional (default=0.5)
+        Threshold value for mapping y_pred_prob to y_pred
+        Note that for threshold ">" is used instead of  ">="
+    average_method: str, optional (default="binary")
+        Method to calculate the average of the metric. Possible values are
+        "micro", "macro", "weighted", "binary"
+    precision_digits: int, optional (default=3)
+        The number of precision digits to format the scores' dataframe
+    display_df: boolean, optional (default=True)
+        Flag to display the formatted scores' dataframe
+    Attributes
+    ----------
+    y_true: numpy.array[int] or list[int]
+        List of predicted class with binary values [0, 1]
+    accuracy: float value between 0. and 1.
+        Classification accuracy based on threshold value
+    balanced_accuracy: float value between 0. and 1.
+        Balanced classification accuracy based on threshold value
+        considering the prevalence of the classes
+    fpr_list: numpy.array[float] or list[float]
+        List of calculated false-positive-rates based on roc_thresholds.
+        This can be used for ROC curve plotting
+    tpr_list: numpy.array[float] or list[float]
+        List of calculated true-positive-rates based on roc_thresholds
+        This can be used for ROC curve plotting
+    roc_thresholds: numpy.array[float] or list[float]
+        List of thresholds value to calculate fpr_list and tpr_list
+    auc_roc: float value between 0. and 1.
+        Area under ROC curve
+    precision_list: numpy.array[float] or list[float]
+        List of calculated precision based on pr_thresholds
+        This can be used for ROC curve plotting
+    recall_list: numpy.array[float] or list[float]
+        List of calculated recall based on pr_thresholds
+        This can be used for ROC curve plotting
+    pr_thresholds: numpy.array[float] or list[float]
+        List of thresholds value to calculate precision_list and recall_list
+    auc_pr: float value between 0. and 1.
+        Area under Precision-Recall curve
+    precision: float value between 0. and 1.
+        Precision based on threshold value
+    recall: float value between 0. and 1.
+        Recall based on threshold value
+    f1: float value between 0. and 1.
+        F1-score based on threshold value (beta=1.0)
+    f2: float value between 0. and 1.
+        F2-score based on threshold value (beta=2.0)
+    f05: float value between 0. and 1.
+        F(1/2)-score based on threshold value (beta=0.5)
+    average_precision: float value between 0. and 1.
+        Avearge precision based on threshold value and class prevalence
+    tn: integer
+        True negative counts based on threshold value
+    fp: integer
+        False positive counts based on threshold value
+    fn: integer
+        False negative counts based on threshold value
+    tp: integer
+        True positive counts based on threshold value
+    threat_score: float value between 0. and 1.
+        Threat score based on threshold value
+    youden_threshold: float value between 0. and 1.
+        Threshold calculated based on Youden Index
+    sens_spec_threshold: float value between 0. and 1.
+        Threshold calculated based on maximized sensitivity-specificity
+    prec_rec_threshold: float value between 0. and 1.
+        Threshold calculated based on maximized precision-recall
+    thresholds_dict: dict()
+        Dictionary of all calculated thresholds
+    metrics_dict: dict()
+        Dictionary of all calculated metrics
+    metrics_df: Pandas DataFrame()
+        Pandas DataFrame of all calculated metrics with threshold as index
+    average_methods: list[str]
+        List of all possible average methods
+    plotting_dict: dict()
+        Plotting object as a dictionary consists of all
+        calculated metrics which was used to plot the thresholds
     """
 
     def __init__(
         self,
         y_true,
         y_pred_proba,
-        threshold=0.5,
-        average_method="binary",
-        precision_digits=3,
-        figsize=(12, 12),
+        threshold=None,
+        average_method=None,
+        precision_digits=None,
         display_df=True,
     ):
-        """
-        Default constructor
-        """
         if not isinstance(y_true, np.ndarray):
             self.y_true = np.array(y_true)
         else:
@@ -68,13 +131,18 @@ class BinaryClassificationMetrics:
             self.y_pred_proba = np.array(y_pred_proba)
         else:
             self.y_pred_proba = y_pred_proba
-        self.threshold = threshold
-        if average_method == "binary":
+        if threshold is None:
+            self.threshold = 0.5
+        else:
+            self.threshold = threshold
+        if average_method == "binary" or average_method is None:
             self.average_method = None
         else:
             self.average_method = average_method
-        self.precision_digits = precision_digits
-        self.figsize = figsize
+        if precision_digits is None:
+            self.precision_digits = 3
+        else:
+            self.precision_digits = precision_digits
         self.display_df = display_df
         self.y_pred = (self.y_pred_proba > self.threshold).astype(int)
         self.accuracy = self._accuracy()
@@ -104,6 +172,8 @@ class BinaryClassificationMetrics:
             self.prec_rec_threshold,
         ) = self._threshold_prec_rec()
         self.thresholds_dict = self._thresholds_dict()
+        self.plotting_dict = self._plotting_dict()
+        self.average_methods = self._average_methods()
 
     def _accuracy(self):
         """
@@ -239,14 +309,14 @@ class BinaryClassificationMetrics:
             w = self.tp + self.tn
             wp = self.tp / w
             wn = self.tn / w
-            threat_score = wp * (
-                self.tp / (self.tp + self.fp + self.fn)
-            ) + wn * (self.tn / (self.tn + self.fn + self.fp))
+            threat_score = wp * (self.tp / (self.tp + self.fp + self.fn)) + wn * (
+                self.tn / (self.tn + self.fn + self.fp)
+            )
 
         elif self.average_method == "macro":
-            threat_score = 0.5 * (
-                self.tp / (self.tp + self.fp + self.fn)
-            ) + 0.5 * (self.tn / (self.tn + self.fn + self.fp))
+            threat_score = 0.5 * (self.tp / (self.tp + self.fp + self.fn)) + 0.5 * (
+                self.tn / (self.tn + self.fn + self.fp)
+            )
 
         else:
             threat_score = self.tp / (self.tp + self.fp + self.fn)
@@ -259,9 +329,7 @@ class BinaryClassificationMetrics:
         precision digits and average method"""
         metrics_dict = {
             "Accuracy": round(self.accuracy, self.precision_digits),
-            "Balanced Accuracy": round(
-                self.balanced_accuracy, self.precision_digits
-            ),
+            "Balanced Accuracy": round(self.balanced_accuracy, self.precision_digits),
             "ROC AUC": round(self.auc_roc, self.precision_digits),
             "PR AUC": round(self.auc_pr, self.precision_digits),
             "Precision": round(self.precision, self.precision_digits),
@@ -270,9 +338,7 @@ class BinaryClassificationMetrics:
             "F-2 Score": round(self.f2, self.precision_digits),
             "F-0.50 Score": round(self.f05, self.precision_digits),
             "Threat Score": round(self.threat_score, self.precision_digits),
-            "Average Precision": round(
-                self.average_precision, self.precision_digits
-            ),
+            "Average Precision": round(self.average_precision, self.precision_digits),
             "TP": self.tp,
             "TN": self.tn,
             "FP": self.fp,
@@ -334,9 +400,7 @@ class BinaryClassificationMetrics:
 
         if self.display_df:
             display(
-                metrics_df.style.background_gradient(cmap=cm).set_table_styles(
-                    styles
-                )
+                metrics_df.style.background_gradient(cmap=cm).set_table_styles(styles)
             )
 
         return metrics_df
@@ -380,193 +444,45 @@ class BinaryClassificationMetrics:
 
         return thresholds_dict
 
-    def _plot_thresholds(self):
+    def _plotting_dict(self):
         """
-        Function to plot all the calculated thresholds
+        Function to return the plotting properties as a dictionary
         """
+        plotting_dict = {
+            "roc_thresholds": self.roc_thresholds,
+            "pr_thresholds": self.pr_thresholds,
+            "precision_list": self.precision_list,
+            "recall_list": self.recall_list,
+            "y_pred_proba": self.y_pred_proba,
+            "y_true": self.y_true,
+            "fpr_list": self.fpr_list,
+            "tpr_list": self.tpr_list,
+            "auc_roc": self.auc_roc,
+            "youden_index": self.youden_index,
+            "youden_threshold": self.youden_threshold,
+            "sens_spec_threshold": self.sens_spec_threshold,
+            "prec_rec_threshold": self.prec_rec_threshold,
+            "auc_pr": self.auc_pr,
+            "prec_rec_index": self.prec_rec_index,
+        }
 
-        # prepare thresholds for plotting
-        thr_set1 = np.arange(
-            min(self.roc_thresholds), max(self.roc_thresholds), 0.01
-        )
-        thr_set2 = np.arange(
-            min(self.pr_thresholds), max(self.pr_thresholds), 0.01
-        )
-        f1_list = [
-            2
-            * (self.precision_list[i] * self.recall_list[i])
-            / (self.precision_list[i] + self.recall_list[i])
-            for i in range(len(self.precision_list))
-        ]
-        queue_rate_list = [
-            (self.y_pred_proba >= thr).mean() for thr in thr_set2
-        ]
-        accuracy_list = [
-            accuracy_score(self.y_true, (self.y_pred_proba >= thr).astype(int))
-            for thr in thr_set1
-        ]
+        return plotting_dict
 
-        # subplot 1: roc curve
-        plt.figure(figsize=self.figsize)
-        plt.subplot(2, 2, 1)
-        plt.plot(
-            self.fpr_list,
-            self.tpr_list,
-            color="red",
-            label=f"AUC = {self.auc_roc:.3f}",
-        )
-        plt.plot(
-            self.fpr_list[self.youden_index],
-            self.tpr_list[self.youden_index],
-            marker="o",
-            color="navy",
-            ms=10,
-        )
-        plt.xlim([-0.01, 1.01])
-        plt.ylim([-0.01, 1.01])
-        plt.xlabel("1 - Specificity", fontsize=12)
-        plt.ylabel("Sensitivity", fontsize=12)
-        plt.tick_params(axis="both", which="major", labelsize=12)
-        plt.legend(prop={"size": 12}, loc=0)
-        plt.title("ROC Curve", fontsize=12)
-        plt.annotate(
-            f"Threshold = {self.youden_threshold:.3f}",
-            xy=(
-                self.fpr_list[self.youden_index],
-                self.tpr_list[self.youden_index],
-            ),
-            xycoords="data",
-            xytext=(
-                self.fpr_list[self.youden_index] + 0.4,
-                self.tpr_list[self.youden_index] - 0.4,
-            ),
-            arrowprops=dict(facecolor="black", shrink=0.05),
-            horizontalalignment="right",
-            verticalalignment="bottom",
-        )
-
-        # subplot 2: preferred scores vs thresholds
-        plt.subplot(2, 2, 2)
-        plt.plot(self.roc_thresholds, 1 - self.fpr_list, label="Specificity")
-        plt.plot(self.roc_thresholds, self.tpr_list, label="Sensitivity")
-        plt.plot(thr_set1, accuracy_list, label="Accuracy")
-        plt.xlabel("Threshold", fontsize=12)
-        plt.ylabel("Score", fontsize=12)
-        plt.tick_params(axis="both", which="major", labelsize=12)
-        plt.legend(bbox_to_anchor=(1.2, 0.5), loc="center", ncol=1)
-        plt.axvline(self.sens_spec_threshold, color="k", ls="--")
-        plt.title(f"Threshold = {self.sens_spec_threshold:.3f}", fontsize=12)
-        plt.title("Preferred Scores vs Thresholds", fontsize=12)
-        plt.xlim([-0.01, 1.01])
-        plt.ylim([-0.01, 1.01])
-        if self.sens_spec_threshold <= 0.5:
-            plt.annotate(
-                f"Threshold = {self.sens_spec_threshold:.3f}",
-                xy=(self.sens_spec_threshold, 0.05),
-                xycoords="data",
-                xytext=(self.sens_spec_threshold + 0.1, 0.05),
-                arrowprops=dict(facecolor="black", shrink=0.05),
-                horizontalalignment="left",
-                verticalalignment="bottom",
-            )
-        else:
-            plt.annotate(
-                f"Threshold = {self.sens_spec_threshold:.3f}",
-                xy=(self.sens_spec_threshold, 0.05),
-                xycoords="data",
-                xytext=(self.sens_spec_threshold - 0.4, 0.05),
-                arrowprops=dict(facecolor="black", shrink=0.05),
-                horizontalalignment="left",
-                verticalalignment="bottom",
-            )
-
-        # subplot 3: precision-recall curve
-        plt.subplot(2, 2, 3)
-        plt.plot(
-            self.recall_list,
-            self.precision_list,
-            color="red",
-            label=f"PR AUC ={self.auc_pr:.3f}",
-        )
-        plt.plot(
-            self.recall_list[self.prec_rec_index],
-            self.precision_list[self.prec_rec_index],
-            marker="o",
-            color="navy",
-            ms=10,
-        )
-        plt.axvline(
-            x=self.recall_list[self.prec_rec_index],
-            ymin=self.recall_list[self.prec_rec_index],
-            ymax=self.precision_list[self.prec_rec_index],
-            color="navy",
-            ls="--",
-        )
-        plt.xlim([-0.01, 1.01])
-        plt.ylim([-0.01, 1.01])
-        plt.xlabel("Recall", fontsize=12)
-        plt.ylabel("Precision", fontsize=12)
-        plt.tick_params(axis="both", which="major", labelsize=12)
-        plt.legend(prop={"size": 12}, loc=0)
-        plt.title("Precision-Recall Curve", fontsize=12)
-        plt.annotate(
-            f"Threshold = {self.prec_rec_threshold:.3f}",
-            xy=(
-                self.recall_list[self.prec_rec_index],
-                self.precision_list[self.prec_rec_index],
-            ),
-            xycoords="data",
-            xytext=(
-                self.recall_list[self.prec_rec_index] - 0.4,
-                self.precision_list[self.prec_rec_index] - 0.4,
-            ),
-            arrowprops=dict(facecolor="black", shrink=0.05),
-            horizontalalignment="left",
-            verticalalignment="bottom",
-        )
-
-        # subplot 4: preferred Scores vs Thresholds
-        plt.subplot(2, 2, 4)
-        plt.plot(
-            self.pr_thresholds, self.precision_list[1:], label="Precision"
-        )
-        plt.plot(self.pr_thresholds, self.recall_list[1:], label="Recall")
-        plt.plot(self.pr_thresholds, f1_list[1:], label="F1-Score")
-        plt.plot(thr_set2, queue_rate_list, label="Queue Rate")
-        plt.xlabel("Threshold", fontsize=12)
-        plt.ylabel("Score", fontsize=12)
-        plt.tick_params(axis="both", which="major", labelsize=12)
-        plt.axvline(self.prec_rec_threshold, color="k", ls="--")
-        plt.title("Preferred Scores vs Thresholds", fontsize=12)
-        plt.xlim([-0.01, 1.01])
-        plt.ylim([-0.01, 1.01])
-        plt.legend(bbox_to_anchor=(1.2, 0.5), loc="center", ncol=1)
-        if self.prec_rec_threshold <= 0.5:
-            plt.annotate(
-                f"Threshold = {self.prec_rec_threshold:.3f}",
-                xy=(self.prec_rec_threshold, 0.03),
-                xycoords="data",
-                xytext=(self.prec_rec_threshold + 0.1, 0.03),
-                arrowprops=dict(facecolor="black", shrink=0.05),
-                horizontalalignment="left",
-                verticalalignment="bottom",
-            )
-        else:
-            plt.annotate(
-                f"Threshold = {self.prec_rec_threshold:.3f}",
-                xy=(self.prec_rec_threshold, 0.03),
-                xycoords="data",
-                xytext=(self.prec_rec_threshold - 0.4, 0.03),
-                arrowprops=dict(facecolor="black", shrink=0.05),
-                horizontalalignment="left",
-                verticalalignment="bottom",
-            )
-
-        plt.show()
-
-    @staticmethod
-    def _average_methods():
+    def _average_methods(self):
         """
         Function to return average methods as a list
         """
         return ["binary", "weighted", "macro", "micro"]
+
+    def plot(self, figsize=None):
+        """
+        Function to plot binary classification metrics.
+        This function is a helper function based on the plotting_dict
+        attribute of the BinaryClassificationMetrics class.
+        Parameters
+        ----------
+        figsize: tuple, optional, (default=(12, 12))
+            Figure size
+        """
+
+        plot_binary_classification_metrics(figsize, **self.plotting_dict)
